package se.chalmers.cs.gf.abssyn;

/**
 *  A node in abstract syntax.
 */
public class Fun implements Tree {

        private String label;

        private Tree[] children;

        public Fun(String label) {
                this(label, new Tree[0]);
        }

        public Fun(String label, Tree[] children) {
                this.label = label;
                this.children = children;
        }

        public boolean isLiteral() {
                return false;
        }

        public String getLabel() {
                return label;
        }

        public int countChildren() {
                return children.length;
        }

        public Tree getChild(int i) {
                return children[i];
        }

        public Tree[] getChildren() {
                return children;
        }

        public boolean equals(Object o) {
                return o instanceof Fun && equals((Fun)o);
        }

        public boolean equals(Fun f) {
                if (!label.equals(f.label))
                        return false;
                if (children.length != f.children.length)
                        return false;
                for (int i = 0; i < children.length; i++)
                        if (!children[i].equals(f.children[i]))
                                return false;
                return true;
        }

        public int hashCode() {
                int c = label.hashCode();
                for (int i = 0; i < children.length; i++)
                        // FIXME: is this a good formula?
                        c += children[i].hashCode() * (i+1); 
                return c;
        }

        public String toString() {
                StringBuilder sb = new StringBuilder(label);
                for (Tree c : children) {
                        sb.append(' ');
                        if (c.isLiteral() || (c instanceof Fun && ((Fun)c).getChildren().length == 0)) {
                                sb.append(c.toString());
                        } else {
                                sb.append('(').append(c.toString()).append(')');
                        }
                }
                return sb.toString();
        }

        public <R,A> R accept(TreeVisitor<R,A> v, A arg) {
                return v.visit(this, arg); 
        }

}
