package se.chalmers.cs.gf.oaa;

import com.sri.oaa2.icl.*;

import se.chalmers.cs.gf.abssyn.*;

import java.util.*;


/**
 *  Functions are converted to ICL structs, metavariables become 
 *  ICL variables, string literals become gf_str(string value) and
 *  integer literals gf_int(int value). The latter two are there to 
 *  avoid confusion between nullary functions and literals.
 */
public class TreeToIclTerm {

        public static IclTerm treeToIclTerm(Tree t) {
                return t.accept(new ToIclTermVisitor(), null);
        }

        private static class ToIclTermVisitor implements TreeVisitor<IclTerm,Object> {

                public IclTerm visit(Fun f, Object arg) {
                        String functor = f.getLabel().toLowerCase();

                        // hack to get around problems with nullary structs
                        // a struct with no children is rendered as functor()
                        // but OAA doesn't like that:
                        // "Syntax error in read/2
                        // ) cannot start an expression"
                        if (f.countChildren() == 0)
                                return new IclStr(functor);

                        List<IclTerm> l = new LinkedList<IclTerm>();
                        for (Tree c : f.getChildren())
                                l.add(c.accept(this, arg));
                        return new IclStruct(functor, l);
                }

                public IclTerm visit(IntLiteral l, Object arg) {
                        return new IclStruct("gf_int", new IclInt(l.getValue()));
//                        return new IclInt(l.getValue());
                }

                public IclTerm visit(StringLiteral l, Object arg) {
                        return new IclStruct("gf_str", new IclStr(l.getValue()));
//                        IclStr s = new IclStr(l.getValue());
//                        s.getForcedQuoted(true);
//                        return s;
                }

                public IclTerm visit(MetaVariable v, Object arg) {
                        return new IclVar(); // FIXME: include type in variable name?
                }

        }

}
