package se.chalmers.cs.gf.parse.chart;

import se.chalmers.cs.gf.abssyn.Tree;
import se.chalmers.cs.gf.parse.Parser;
import static se.chalmers.cs.gf.parse.ParseLogger.*;

import java.util.Collection;
import java.util.List;


/**
 *  Combines lexing, kilbury parsing and tree building.
 */
public class ChartParser implements Parser {

        private CFGrammar grammar;

        private KilburyParser parser;

        public ChartParser(CFGrammar grammar) {
                this.grammar = grammar;
                parser = new KilburyParser(grammar);

                fine("Created chart parser for " + grammar.getName());
        }

        public Collection<Tree> parse(String str) {
                return parse(str, grammar.getStart());
        }

        public Collection<Tree> parse(String str, String cat) {
                return parse(str, new Category(cat));
        }

        public Collection<Tree> parse(String str, Category cat) {
                fine("Parsing \"" + str + "\" as " + cat + ".");

                Lexer l = new Lexer();
                List<Token> tokens = l.lex(str);

                fine("Tokens: " + tokens);

                Chart chart = parser.parse(tokens);

                fine("Finished parsing.");

                finest(chartInfo(chart));

                fine("Building trees.");

                Collection<Tree> trees = new TreeBuilder(chart).buildTree(cat);

                fine("Built " + trees.size() + " trees");

                return trees;
        }

        private static void ln(StringBuilder a, CharSequence s) {
                a.append(s).append('\n');
        }

        private String chartInfo(Chart chart) {
                StringBuilder s = new StringBuilder();
                ln(s, "Passive edges:");
                ln(s, chart.toString());

                ln(s, "Passive non-empty edges: ");
                int n = chart.getSize();
                for (int i = 0; i <= n; i++)
                        for (int j = i+1; j <= n; j++)
                                for (Edge e : chart.get(i,j))
                                        ln(s, "("+i+","+j+"): " + e);

                ln(s, "Complete parses:");
                for (Edge e : chart.get(0, chart.getSize()))
                        ln(s, e.toString());

                return s.toString();
        }

        public String getName() {
                return grammar.getName();
        }

        public boolean tryThisOne() {
                return true;
        }

}
