package se.chalmers.cs.gf.parse.chart;

import java.util.*;

/**
 *  A CF grammar prodcution whose right hand side is a list of non-terminals.
 */
public class NonTerminalRule extends Rule {

        private List<Category> rhs;

        private String fun;

        private int[][] profile;

        public NonTerminalRule(String name, Category cat, List<Category> rhs, 
                               String fun, int[][] profile) {
                super(name, cat);
                this.rhs = rhs;
                this.fun = fun;
                this.profile = profile;
        }

        /**
         * Get the right hand side of the rule.
         * Do not modify the returned list.
         */
        public List<Category> getRhs() {
                return rhs;
        }

        public String getFun() {
                return fun;
        }

        public int[][] getProfile() {
                return profile;
        }

        /**
         *  Checks if the right hand side of rule starts with the given 
         *  non-terminal.
         */
        public boolean startsWith(Category c) {
                return rhs.size() > 0 && rhs.get(0).equals(c);
        }

        public int getSize() {
                return rhs.size();
        }

        /**
         *  Checks if the right hand side of this rule consists of a 
         *  single terminal.
         */
        public boolean isTerminal() {
                return false;
        }

        public boolean equals(Object o) {
                return o instanceof NonTerminalRule && equals((NonTerminalRule)o);
        }

        public boolean equals(NonTerminalRule r) {
                return getName().equals(r.getName()) && Arrays.deepEquals(profile, r.profile);
        }

        public int hashCode() {
                return getName().hashCode() + Arrays.deepHashCode(profile);
        }

        public String toString() {
                StringBuilder sb = new StringBuilder();
                sb.append(getName());

                sb.append("[");
                for (int i = 0; i < profile.length; i++) {
                        sb.append("[");
                        for (int j = 0; j < profile[i].length; j++) {
                                sb.append(profile[i][j]);
                                if (j < profile[i].length-1)
                                        sb.append(",");
                        }                        
                        sb.append("]");
                        if (i < profile.length-1)
                                sb.append(",");
                }
                sb.append("]");
                sb.append(". ");

                sb.append(getCategory());
                sb.append(" -> ");
                for (Category c : getRhs())
                        sb.append(c).append(" ");

                return sb.toString();
        }

}
