package se.chalmers.cs.gf.util;

import se.chalmers.cs.gf.GFException;

import java.io.*;
import java.util.Properties;

/**
 *  Some utilties for files and property files.
 */
public class FileUtil {

        public static Reader openFile(String filename, String charset) 
                throws IOException, FileNotFoundException {

                return new InputStreamReader(new FileInputStream(filename), charset);
        }

        public static Reader openFile(String filename) 
                throws IOException, FileNotFoundException {

                return openFile(filename, "UTF-8");
        }

        public static Reader readResource(Class<?> cls, String name) throws IOException {
                return readResource(cls, name, "UTF-8");
        }

        public static Reader readResource(Class<?> cls, String name, String charset) throws IOException {
                InputStream in = cls.getResourceAsStream(name);
                if (in == null)
                        throw new FileNotFoundException("Resource " + name + " not found.");
                return new InputStreamReader(in, charset);
        }

        public static Properties readPropertiesResource(Class<?> cls, String name) throws IOException {
                Properties prop = new Properties();
                InputStream propIn = cls.getResourceAsStream(name);
                if (propIn == null)
                        throw new FileNotFoundException("Resource " + name + " not found.");
                prop.load(propIn);
                propIn.close();
                return prop;
        }

        public static String getProp(Properties prop, String name) {
                String value = prop.getProperty(name);
                if (value == null)
                        throw new GFException("Property " + name + " is not set");
                return value;
        }

        public static String[] getListProp(Properties prop, String name) {
                return StringUtil.splitAtCommas(prop.getProperty(name));
        }

}
